"""
HTTP convenience commands
"""

import json
from contextlib import suppress
from pprint import pprint
from time import sleep

import click
import requests
import urllib3
from brewblox_ctl import click_helpers, utils
from requests.exceptions import ConnectionError, HTTPError

RETRY_COUNT = 60
RETRY_INTERVAL_S = 10
METHODS = ['wait', 'get', 'put', 'post', 'patch', 'delete']


def wait(url, info_updates=False):
    echo = utils.info if info_updates else click.echo
    for i in range(RETRY_COUNT):
        with suppress(ConnectionError, HTTPError):
            echo(f'Connecting {url}, attempt {i+1}/{RETRY_COUNT}')
            requests.get(url, verify=False).raise_for_status()
            echo('Success!')
            return

        sleep(RETRY_INTERVAL_S)
    raise TimeoutError('Retry attempts exhausted')


@click.group(cls=click_helpers.OrderedGroup)
def cli():
    """Click group and entrypoint"""


@cli.command(hidden=True)
@click.argument('method', required=True, type=click.Choice(METHODS))
@click.argument('url', required=True)
@click.option('--json-body', type=click.BOOL, default=True, help='Set JSON content headers.')
@click.option('-f', '--file', help='Load file and send as body.')
@click.option('-d', '--data', help='Request body.')
@click.option('-H', '--header', multiple=True, type=(str, str))
@click.option('-p', '--param', multiple=True, type=(str, str))
@click.option('-q', '--quiet', is_flag=True, help='Do not print the response. Takes precedence over --pretty.')
@click.option('--pretty', is_flag=True, help='Pretty-print JSON response.')
@click.option('--allow-fail', is_flag=True, help='Do not throw on HTTP errors.')
def http(method, url, json_body, file, data, header, param, quiet, pretty, allow_fail):
    """Send HTTP requests"""
    urllib3.disable_warnings()

    if method == 'wait':
        return wait(url)

    body = None
    kwargs = {
        'headers': dict(header),
        'params': dict(param),
        'verify': False,
    }

    if file and data:
        raise ValueError('Unable to process both file and data body')
    elif file:
        with open(file) as f:
            body = f.read()
    elif data:
        body = data

    if body:
        if json_body:
            kwargs['json'] = json.loads(body)
        else:
            kwargs['data'] = body

    resp = getattr(requests, method)(url, **kwargs)
    try:
        if not quiet:
            if pretty and json_body:
                pprint(resp.json())
            else:
                click.echo(resp.text)
        resp.raise_for_status()
    except Exception as ex:
        if not allow_fail:
            click.echo(f'Error: {ex}')
            raise SystemExit(1)



"""
Brewblox-ctl docker commands
"""


import click
from brewblox_ctl import click_helpers, utils
from brewblox_ctl.utils import sh


@click.group(cls=click_helpers.OrderedGroup)
def cli():
    """Command collector"""


@cli.command(context_settings=dict(
    ignore_unknown_options=True,
))
@click.argument('compose_args', nargs=-1, type=click.UNPROCESSED)
def up(compose_args):
    """Start all services.

    This wraps `docker-compose up -d`
    """
    utils.check_config()
    utils.confirm_mode()
    sudo = utils.optsudo()
    sh(f'{sudo}docker-compose up -d ' + ' '.join(list(compose_args)))


@cli.command(context_settings=dict(
    ignore_unknown_options=True,
))
@click.argument('compose_args', nargs=-1, type=click.UNPROCESSED)
def down(compose_args):
    """Stop all services.

    This wraps `docker-compose down`
    """
    utils.check_config()
    utils.confirm_mode()
    sudo = utils.optsudo()
    sh(f'{sudo}docker-compose down ' + ' '.join(list(compose_args)))


@cli.command(context_settings=dict(
    ignore_unknown_options=True,
))
@click.argument('compose_args', nargs=-1, type=click.UNPROCESSED)
def restart(compose_args):
    """Recreates all services.

    This wraps `docker-compose up -d --force-recreate`

    Note: `docker-compose restart` also exists -
    it restarts containers without recreating them.
    """
    utils.check_config()
    utils.confirm_mode()
    sudo = utils.optsudo()
    sh(f'{sudo}docker-compose up -d --force-recreate ' + ' '.join(list(compose_args)))


@cli.command()
@click.argument('services', nargs=-1, required=False)
def follow(services):
    """Show logs for one or more services.

    This will start watching the logs for specified services.
    Call without arguments to show logs for all running services.

    Once started, press ctrl+C to stop.

    Service name will be equal to those specified in docker-compose.log,
    not the container name.

    To follow logs for service 'spark-one':

    \b
        GOOD: `brewblox-ctl follow spark-one`
         BAD: `brewblox-ctl follow brewblox_spark-one_1`
    """
    utils.check_config()
    sudo = utils.optsudo()
    sh(f'{sudo}docker-compose logs --follow ' + ' '.join(services))


@cli.command()
def kill():
    """Stop and remove all containers on this computer.

    This includes those not from Brewblox.
    """
    utils.confirm_mode()
    sudo = utils.optsudo()
    sh(f'{sudo}docker rm --force $({sudo}docker ps -aq)', check=False)



"""
Const values
"""
import sys

PY = sys.executable

CFG_VERSION_KEY = 'BREWBLOX_CFG_VERSION'
LIB_RELEASE_KEY = 'BREWBLOX_CTL_LIB_RELEASE'
RELEASE_KEY = 'BREWBLOX_RELEASE'
SKIP_CONFIRM_KEY = 'BREWBLOX_SKIP_CONFIRM'
DEBUG_KEY = 'BREWBLOX_DEBUG'

LOG_SHELL = 'SHELL'.ljust(10)
LOG_PYTHON = 'PYTHON'.ljust(10)
LOG_ENV = 'ENV'.ljust(10)
LOG_COMPOSE = 'COMPOSE'.ljust(10)
LOG_INFO = 'INFO'.ljust(10)
LOG_WARN = 'WARN'.ljust(10)
LOG_ERR = 'ERROR'.ljust(10)
